<?php
/**
 * Plugin implementation of the Ephoto Dam field type
 *
 * FieldType(
 *   id = "ephoto_dam_field",
 *   label = @Translation("Ephoto Dam Field"),
 *   description = @Translation("Stores a Ephoto 
 *   Dam URL and then outputs some embed code"),
 *   category = @Translation("Media"),
 *   default_widget = "ephoto_dam_field_widget",
 *   default_formatter = "ephoto_dam_field_formatter",
 *   constraints = {"EphotoDamFieldValidation" = {}} 
 * )
 * PHP version 8
 * 
 * @category Field
 * @package  Ephoto-Drupal
 * @author   Einden <contact@einden.fr>
 * @license  https://www.ephoto.fr/ Einden
 * @link     https://www.ephoto.fr/
 */
namespace Drupal\ephoto_dam_field\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\Core\TypedData\TraversableTypedDataInterface;


 /**
  * EphotoDam field
  * 
  * @category Field
  * @package  Ephoto-Drupal
  * @author   Einden <contact@einden.fr>
  * @license  https://www.ephoto.fr/ Einden
  * @link     https://www.ephoto.fr/
  */
class EphotoDamField extends FieldItemBase
{

    /**
     * Default settings
     *
     * @const
     */
    private const DEFAULT_SETTINGS = [
        'version_support' => false,
        'captions_format' => ''
    ];   

    /**
     * Construct function of a field
     * 
     * @param mixed                         $definition The definition of the field
     * @param string                        $name       The name of the field
     * @param TraversableTypedDataInterface $parent     The parent of the field
     * 
     *                                                  {@inheritdoc}
     */
    public function __construct($definition, 
        $name = null, 
        TraversableTypedDataInterface $parent = null
    ) {
        parent::__construct($definition, $name, $parent);
    }

    /**
     * Create the instante of a field
     * 
     * @param mixed                         $definition The definition of the field
     * @param string                        $name       The name of the field
     * @param TraversableTypedDataInterface $parent     The parent of the field
     *                                                  {@inheritdoc}
     * 
     * @return EphotoDamField
     */
    public static function createInstance(
        $definition, 
        $name = null, 
        TraversableTypedDataInterface $parent = null
    ) {
        return new static($definition, $name, $parent);
    }


    /**
     * Set the field definition
     * 
     * @param FieldStorageDefinitionInterface $field_definition 
     *                                                          The definition 
     *                                                          of the field
     * 
     * @return array
     */
    public static function schema(
        FieldStorageDefinitionInterface $field_definition
    ) {
        return [
            'columns' => [
                'identifier' => [
                    'type' => 'int',
                    'unsigned' => true,
                ],                
                'url' => [
                    'type' => 'varchar',
                    'length' => 256
                ],
                'image_size' => [
                    'type' => 'varchar',
                    'length' => 9
                ],
                'version' => [
                    'type' => 'varchar',
                    'length' => 20 
                ],
                'caption' => [
                    'type' => 'varchar',
                    'length' => 256
                ],
                'thumbnail' => [
                    'type' => 'varchar',
                    'length' => 256
                ]            
            ]
        ];
    } 

    /**
     * Set the properties of the field definition
     * 
     * @param FieldStorageDefinitionInterface $field_definition 
     *                                                          The definition 
     *                                                          of the field
     * 
     * @return array
     */
    public static function propertyDefinitions(
        FieldStorageDefinitionInterface $field_definition
    ) {
        $properties['identifier'] = DataDefinition::create('integer')
        ->setLabel(t('File identifier'));  
          
        $properties['url'] = DataDefinition::create('string')
        ->setLabel(t('Url'));

        $properties['image_size'] = DataDefinition::create('string')
        ->setLabel(t('Image size'));

        $properties['version'] = DataDefinition::create('string')
        ->setLabel(t('Version'));

        $properties['caption'] = DataDefinition::create('string')
        ->setLabel(t('Caption'));
            
        $properties['thumbnail'] = DataDefinition::create('string')
        ->setLabel(t('Thumbnail'));
    
        return $properties;
    }

    /**
     * Return if the field is empty
     * 
     * @return boolean
     */
    public function isEmpty() 
    {
        $url = $this->get('url')->getValue();
        $image_size = $this->get('image_size')->getValue();
        $version = $this->get('version')->getValue();

        return empty($url) && empty($image_size) && empty($version);
    }

    /**
     * Set the properties of the field definition
     * 
     * @param array $form       
     *                          The form of the field
     * @param array $form_state 
     *                          The state of the 
     *                          form of the field
     * 
     * @return array
     */
    public function fieldSettingsForm(
        array $form, 
        FormStateInterface $form_state
    ) {
        $field_name = $this->getParent()->getName();

        $form = [];
 
        $form['version_support'] = [
                '#title' => t('Version support'),
                '#description' => t(
                    'An url will be automatically associated 
                    with each version with the same label in Ephoto Dam'
                ) 
                . '.',
                '#type' => 'checkbox',
                '#default_value' => $this->getSetting('version_support'),
                '#attributes' => [
                    'class' => ['ephoto-dam-field-settings-version-support'],
                    'onclick' => 'EphotoDamField[\'' 
                    . $field_name 
                    . '\'].enableVersionSupport(this)'
                ]
        ];

        $form['captions_format'] = [
            '#title' => t('Captions format'),
            '#type' => 'textarea',
            '#default_value' => $this->getSetting('captions_format'),
            '#description' => t('Please format the captions'),
            '#rows' => 1
        ];        

        return $form;
    }

    /**
     * Set the field to the default settings
     * 
     * @return EphotoDamField
     */
    public static function defaultFieldSettings() 
    {
        return self::DEFAULT_SETTINGS;
    }  
}
