<?php 
/**
 * Configuration form for drupal
 * Contains \Drupal\ephoto_dam\Form\ConfigurationForm
 * PHP version 8
 * 
 * @category Ephoto_Pluggin
 * @package  Ephoto-Drupal
 * @author   Einden <contact@einden.fr>
 * @license  https://www.ephoto.fr/ Einden
 * @link     https://www.ephoto.fr/
 */ 

namespace Drupal\ephoto_dam\Form;  

use Drupal\Core\Form\ConfigFormBase;  
use Drupal\Core\Form\FormStateInterface; 

/**
 * Class to create and use the configuration form
 * 
 * @category Configuration_Form_Class
 * @package  Ephoto-Drupal
 * @author   Einden <contact@einden.fr>
 * @license  https://www.ephoto.fr/ Einden
 * @link     https://www.ephoto.fr/
 */
class ConfigurationForm extends ConfigFormBase
{
    /**
     * Returns the array of the editable config names
     * 
     * @return array 
     * {@inheritdoc}
     */ 
    protected function getEditableConfigNames() 
    {
        return [  
            'ephoto_dam.settings',  
        ];  
    }

    /**
     * Returns the form ID
     * 
     * @return string
     * {@inheritdoc}  
     */  
    public function getFormId() 
    {
        return 'ephoto_dam_admin_settings'; 
    }

    /**
     * Builds the form given a form and a form state interface
     * 
     * @param array              $form       the config file form to be set
     * @param FormStateInterface $form_state the form state Interface to be set
     * 
     * @return FormBuilder
     *                                       {@inheritdoc}  
     */  
    public function buildForm(array $form, FormStateInterface $form_state) 
    {
        $config = $this->config('ephoto_dam.settings');  

        $form['settings_server_url'] = [
            '#title' => $this->t('Server URL'),
            '#type' => 'url',  
            '#description' => $this->t(
                'Please enter the URL of your Ephoto Dam software'
            ),  
            '#default_value' => $config->get('server_url'),
            '#required' => true,
            '#attributes' => [
                'placeholder' => $this->t('https://ephoto.mycompany.com/')
            ],
            '#size' => 75,
        ];  

        return parent::buildForm($form, $form_state);  
    } 

    /**
     * Validates the form given a form and a form state interface
     * 
     * @param array              $form       the config file form to be validated
     * @param FormStateInterface $form_state the form state Interface to be validated
     * 
     * @return FormBase
     *                                       {@inheritdoc}
     */
    public function validateForm(array &$form, FormStateInterface $form_state) 
    {
        $server_url = $form_state->getValue('settings_server_url');

        $server_url = filter_var($server_url, FILTER_VALIDATE_URL); 

        if (!$server_url) {
            $form_state->setErrorByName(
                'settings_server_url', t('Please enter a correct URL')
            );
        }

        if (substr($server_url, -1) !== '/') {
            $server_url .= '/';
        }        

        parent::validateForm($form, $form_state);
    } 
    
    /**
     * Submits the form given a form and a form state interface
     * 
     * @param array              $form       the config file form to be submited
     * @param FormStateInterface $form_state the form state Interface to be submited
     * 
     * @return FormBuilder
     *                                       {@inheritdoc}
     */
    public function submitForm(array &$form, FormStateInterface $form_state) 
    {
        $this->config('ephoto_dam.settings')
            ->set(
                'server_url', $form_state->getValue(
                    'settings_server_url'
                )
            )
            ->save();
    
        parent::submitForm($form, $form_state);
    }    
}  